<?php

namespace Slack\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Slack\Client\Payload;
use Slack\Options\Factory as SlackOptionsFactory;

use App\Http\Controllers\Controller;

class InteractiveOptionsController extends Controller
{
	private const LOGKEY = 'CIO';

	/**
	 * Fire slack event
	 *
	 * @param Request $request
	 * @return \Illuminate\Http\Response|\Laravel\Lumen\Http\ResponseFactory
	 */
	public function fire(Request $request)
	{
		$event = SlackOptionsFactory::make(new Payload(json_decode($request->payload,TRUE),TRUE));
		Log::debug(sprintf('%s:Firing Event [%s] and responding [%s]',static::LOGKEY,get_class($event),$event->respondNow));

		if ($event->respondNow) {
			if (! method_exists($event,'respond')) {
				Log::alert(sprintf('%s:Cant respond to Event [%s], no respond method',static::LOGKEY,get_class($event)),['m'=>__METHOD__]);

			} else {
				return ($x=$event->respond())->isEmpty() ? NULL : $x;
			}

		} else {
			event($event);

			Log::info(sprintf('%s:Dispatched Event [%s]',static::LOGKEY,get_class($event)),['m'=>__METHOD__]);
			return response('IO Event Processed',200);
		}
	}
}